# Copyright (C) 2006-2011 Oracle. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, version 2.  This program is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the GNU
# General Public License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 021110-1307, USA.

import os
from glob import glob
from signal import signal, SIGUSR1
from ConfigParser import ConfigParser
from OVSConstants import CONFIG_DIR, PID_DIR
from password import read_password


_g_config = None
_g_passwd = None


def get_config():
    global _g_config
    if not _g_config:
        _g_config = ConfigParser()
        _g_config.read(os.path.join(CONFIG_DIR, "agent.ini"))
    return _g_config


def get_passwd():
    global _g_passwd
    if not _g_passwd:
        _g_passwd = read_password()
        if not _g_passwd:
            raise Exception("No password found in password file.")
    return _g_passwd


def sig_func(signum, frame):
    global _g_config
    global _g_passwd
    if signum == SIGUSR1:
        _g_config = None
        _g_passwd = None


def init_signal_handler():
    signal(SIGUSR1, sig_func)


def get_agent_process():
    result = []
    pidfile_list = glob("%s/*.pid" % PID_DIR)
    for pidfile in pidfile_list:
        pname = os.path.basename(pidfile)[:-4] # remove trailing '.pid'
        f = open(pidfile)
        pid = f.read().strip()
        f.close()
        result.append((pname, pid, pidfile))
    return result


def check_process(pname, pid):
    cmdline_file = '/proc/%s/cmdline' % pid
    if not os.path.exists(cmdline_file):
        raise Exception("Procss %s with PID %s doesn't exist." % (pname, pid))
    f = open(cmdline_file)
    cmdline = f.read()
    f.close()
    if pname not in cmdline:
        raise Exception("Procss with PID %s doesn't match process name %s." % (pid, pname))


def reload_server(name):
    pidfile = ("%s/%s.pid" % (PID_DIR, name))
    f = open(pidfile)
    pid = f.read().strip()
    f.close()
    os.kill(int(pid), SIGUSR1)


def reload_notification_server():
    reload_server('OVSNotificationServer')


def reload_agent_server():
    reload_server('OVSAgentServer')
