# Copyright (C) 2006-2011 Oracle. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, version 2.  This program is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the GNU
# General Public License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 021110-1307, USA.

from OVSDiscoverCommon import SimpleDiscoveryXML, DiscoveryLock

class MountInfo:
    def __init__(self, fs_file, fs_spec, fs_mntops):
        self.fs_file = fs_file
        self.fs_spec = fs_spec
        self.fs_mntops = fs_mntops


def discover_mounted_file_systems(args):
    """
    Get the mounted type file systems.

    Returns an empty document if the server is unassociated.

    @param      None

    @return     Returns the discovery data as an XML document.
                Raises an exception on failure
    """

    lock = DiscoveryLock("discover_mounted_file_systems")
    if not lock.acquire():
        raise Exception("discover_mounted_file_systems already in progress")

    try:
        f = open("/proc/mounts")

    except Exception, e:
        lock.release()
        raise Exception("Unable to discover mounts: %s" % e)

    all_mounts = None

    mounts = {}

    # xml_header

    doc = SimpleDiscoveryXML("Discover_Mounted_File_Systems_Result")

    fstypes = args.split()

    for fstype in fstypes:
        if fstype == 'all':
            all_mounts = 'all'
            break

    try:
        for line in f:

            fs_spec, fs_file, fs_vfstype, fs_mntops, \
                     fs_freq, fs_passno = line.split()
            if not mounts.has_key(fs_vfstype):
                mounts[fs_vfstype] = list()
            mounts[fs_vfstype].append(MountInfo(fs_file, fs_spec, fs_mntops))
    finally:
        lock.release()
        f.close()

    # return all of the mounted files systems or just the specified ones
    if all_mounts == 'all':
        fstypes = mounts.keys()

    for fstype in fstypes:
        mount_list = mounts.get(fstype)
        if mount_list != None:
            doc.push("Filesystem", "Type", fstype)
            for mount_entry in mount_list:
                doc.push("Mount", "Dir", mount_entry.fs_file)
                doc.add("Device", mount_entry.fs_spec)
                doc.add("Mount_Options", mount_entry.fs_mntops)
                doc.pop()
            doc.pop()

    try:
        result = doc.finish()
    except Exception, e:
        lock.release()
        raise Exception("Unable to format discover_mounted_file_systems: %s" %
                        e)
    lock.release()
    return result
