# Copyright (C) 2006-2011 Oracle. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, version 2.  This program is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the GNU
# General Public License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 021110-1307, USA.

import os
from urlgrabber import urlgrab
from urlgrabber.progress import BaseMeter
from OVSAsyncProc import async_proc_prog


class OVSMeter(BaseMeter):
    def __init__(self, update_period, obj, obj_current, obj_total):
        BaseMeter.__init__(self)
        self.update_period = update_period
        self.obj = obj
        self.obj_current = obj_current
        self.obj_total = obj_total

    def send_notification(self, amount_read, now=None):
        if self.size is not None:
            status = {'OPERATION': 'download',
                      'CURRENT_BYTES': long(self.last_amount_read),
                      'TOTAL_BYTES': long(self.size),
                      'OBJECT': self.obj,
                      'CURRENT_OBJECT_COUNT': self.obj_current,
                      'TOTAL_OBJECT_COUNT': self.obj_total}
            async_proc_prog(status)

    def _do_update(self, amount_read, now=None):
        self.send_notification(amount_read, now)

    def _do_end(self, amount_read, now=None):
        self.send_notification(amount_read, now)


def ovs_download_file(url, filename, option=None, obj='file', obj_current=1,
                      obj_total=1, update_period=20):
    meter = OVSMeter(update_period, obj, obj_current, obj_total)
    defaults = {'progress_obj': meter,
                'copy_local': 1,
                'ssl_verify_peer': False,
                'ssl_verify_host': False}
    if option is None:
        option = defaults
    else:
        option.update(defaults)
    try:
        return urlgrab(url, filename, **option)
    except Exception, err:
        try:
            os.unlink(filename)
        except:
            pass
        raise err
