# Copyright (C) 2006-2011 Oracle. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, version 2.  This program is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the GNU
# General Public License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 021110-1307, USA.

import time
import logging
from utils.xmlrpclib import ServerProxy

import OVSDiscover
import OVSDiscoverCluster
import OVSDiscoverHardware
import OVSDiscoverNetwork
import OVSDiscoverPhysicalLuns
import OVSDiscoverServer
import OVSDiscoverServerPool
import OVSVMNetConfig
import OVSDiscoverFSMounts
import OVSRepository
import StoragePluginDiscovery
import StoragePluginManager
import OVSServerPool
import OVSNotificationProxy
import OVSAsyncProc
import OVSIPMI
import OVSCluster
import OVSDownload
import OVSPoolFileSystem
import OVSVirtualIP
import OVSDiscoverPoolFilesystem
import OVSStat
import utils
import vm
import host


def echo(msg):
    return "success:echo=%s" % msg


def sleep(secs):
    secs = float(secs)
    time.sleep(secs)
    return "success:sleep=%f" % secs


def dispatch(uri, func, *args):
    try:
        logging.debug("dispatch function %s to server %s" % (func, uri))
        sp = ServerProxy(uri)
        return getattr(sp, func)(*args)
    except Exception, e:
        logging.error("dispatch error: %s: %s" % (func, e))
        raise


def get_api_version():
    return ['1']


api = {
    'echo': echo,
    'sleep': sleep,
    'dispatch': dispatch,
    'get_api_version': get_api_version,
    'copy_file': utils.common.copy_file,
    'create_vm': vm.create_vm,
    'delete_vm': vm.delete_vm,
    'start_vm': vm.start_vm,
    'stop_vm': vm.stop_vm,
    'reboot_vm': vm.reboot_vm,
    'pause_vm': vm.pause_vm,
    'unpause_vm': vm.unpause_vm,
    'save_vm': vm.save_vm,
    'restore_vm': vm.restore_vm,
    'migrate_vm': vm.migrate_vm,
    'trigger_vm': vm.trigger_vm,
    'sysrq_vm': vm.sysrq_vm,
    'dump_vm_core': vm.dump_vm_core,
    'list_vm_core': vm.list_vm_core,
    'delete_vm_core': vm.delete_vm_core,
    'configure_vm': vm.configure_vm,
    'unconfigure_vm': vm.unconfigure_vm,
    'get_vm_config': vm.get_vm_config,
    'set_vm_config': vm.set_vm_config,
    'list_vm': vm.list_vm,
    'list_vms': vm.list_vms,
    'send_to_guest': vm.send_to_guest,
    'create_template': vm.create_template,
    'delete_template': vm.delete_template,
    'configure_template': vm.configure_template,
    'unconfigure_template': vm.unconfigure_template,
    'get_template_config': vm.get_template_config,
    'set_template_config': vm.set_template_config,
    'import_template': vm.import_template,
    'delete_assembly': vm.delete_assembly,
    'get_assembly_config': vm.get_assembly_config,
    'get_assembly_config_xml': vm.get_assembly_config_xml,
    'import_assembly': vm.import_assembly,
    'deploy_assembly': vm.deploy_assembly,
    'configure_vm_ha': vm.configure_vm_ha,
    'get_xend_config': host.get_xend_config,
    'configure_xend': host.configure_xend,
    'unconfigure_xend': host.unconfigure_xend,
    'restart_xend': host.restart_xend,
    'get_yum_config': host.get_yum_config,
    'configure_yum': host.configure_yum,
    'delete_yum_repo': host.delete_yum_repo,
    'yum_list_package': host.yum_list_package,
    'yum_update': host.yum_update,
    'list_package': host.list_package,
    'install_package': host.install_package,
    'uninstall_package': host.uninstall_package,
    'get_datetime': host.get_datetime,
    'set_datetime': host.set_datetime,
    'get_timezone': host.get_timezone,
    'set_timezone': host.set_timezone,
    'get_ntp': host.get_ntp,
    'set_ntp': host.set_ntp,
    'enable_ntp': host.enable_ntp,
    'disable_ntp': host.disable_ntp,
    'get_log': host.get_log,
    'get_support_files': host.get_support_files,
    'update_agent_password': host.update_agent_password,
    'update_core_api_bindings': host.update_core_api_bindings,
    'notify_manager': OVSNotificationProxy.notify_manager,
    'ovs_download_file': OVSDownload.ovs_download_file,
    'ovs_async_proc': OVSAsyncProc.ovs_async_proc,
    'ovs_async_proc_status': OVSAsyncProc.ovs_async_proc_status,
    'ovs_async_proc_signal': OVSAsyncProc.ovs_async_proc_signal,
    'configure_server_for_cluster': OVSCluster.configure_server_for_cluster,
    'deconfigure_server_for_cluster': OVSCluster.deconfigure_server_for_cluster,
    'update_cluster_configuration': OVSCluster.update_cluster_configuration,
    'create_cluster': OVSCluster.create_cluster,
    'join_cluster': OVSCluster.join_cluster,
    'destroy_cluster': OVSCluster.destroy_cluster,
    'leave_cluster': OVSCluster.leave_cluster,
    'is_cluster_online': OVSCluster.is_cluster_online,
    'set_statistic_interval': OVSStat.set_statistic_interval,
    'status': OVSDiscover.status,
    'discover_cluster': OVSDiscoverCluster.discover_cluster,
    'discover_mounted_file_systems': OVSDiscoverFSMounts.discover_mounted_file_systems,
    'discover_hardware': OVSDiscoverHardware.discover_hardware,
    'discover_network': OVSDiscoverNetwork.discover_network,
    'discover_saved_network': OVSDiscoverNetwork.discover_saved_network,
    'discover_physical_luns': OVSDiscoverPhysicalLuns.discover_physical_luns,
    'discover_pool_filesystem': OVSDiscoverPoolFilesystem.discover_pool_filesystem,
    'discover_repositories': OVSRepository.discover_repositories,
    'discover_server_pool': OVSDiscoverServerPool.discover_server_pool,
    'discover_server': OVSDiscoverServer.discover_server,
    'sys_shutdown': OVSIPMI.sys_shutdown,
    'sys_reboot': OVSIPMI.sys_reboot,
    'remote_power_off': OVSIPMI.remote_power_off,
    'remote_power_on': OVSIPMI.remote_power_on,
    'create_pool_filesystem': OVSPoolFileSystem.create_pool_filesystem,
    'destroy_pool_filesystem': OVSPoolFileSystem.destroy_pool_filesystem,
    'ocfs2_get_meta_data': OVSPoolFileSystem.ocfs2_get_meta_data,
    'mount_repository_fs': OVSRepository.mount_repository_fs,
    'unmount_repository_fs': OVSRepository.unmount_repository_fs,
    'create_repository': OVSRepository.create_repository,
    'add_repository': OVSRepository.add_repository,
    'delete_repository': OVSRepository.delete_repository,
    'get_repository_meta_data': OVSRepository.get_repository_meta_data,
    'synch_repositories': OVSRepository.synch_repositories,
    'discover_repository_db': OVSRepository.discover_repository_db,
    'edit_repository_db': OVSRepository.edit_repository_db,
    'import_virtual_disk': OVSRepository.import_virtual_disk,
    'import_ISO': OVSRepository.import_ISO,
    'delete_virtual_disk': OVSRepository.delete_virtual_disk,
    'delete_ISO': OVSRepository.delete_ISO,
    'create_virtual_disk': OVSRepository.create_virtual_disk,
    'get_last_boot_time': OVSServerPool.get_last_boot_time,
    'take_ownership': OVSServerPool.take_ownership,
    'release_ownership': OVSServerPool.release_ownership,
    'create_server_pool': OVSServerPool.create_server_pool,
    'join_server_pool': OVSServerPool.join_server_pool,
    'destroy_server_pool': OVSServerPool.destroy_server_pool,
    'leave_server_pool': OVSServerPool.leave_server_pool,
    'set_server_pool_alias': OVSServerPool.set_server_pool_alias,
    'update_pool_virtual_ip': OVSServerPool.update_pool_virtual_ip,
    'update_server_roles': OVSServerPool.update_server_roles,
    'set_pool_member_ip_list': OVSServerPool.set_pool_member_ip_list,
    'configure_virtual_ip': OVSVirtualIP.configure_virtual_ip,
    'deconfigure_virtual_ip': OVSVirtualIP.deconfigure_virtual_ip,
    'ovs_br_config': OVSVMNetConfig.ovs_br_config,
    'ovs_async_bridge': OVSVMNetConfig.ovs_async_bridge,
    'ovs_vlan_bridge': OVSVMNetConfig.ovs_vlan_bridge,
    'ovs_bond_config': OVSVMNetConfig.ovs_bond_config,
    'ovs_async_bond': OVSVMNetConfig.ovs_async_bond,
    'ovs_bond_op': OVSVMNetConfig.ovs_bond_op,
    'ovs_bond_mode': OVSVMNetConfig.ovs_bond_mode,
    'ovs_local_config': OVSVMNetConfig.ovs_local_config,
    'ovs_vlan_config': OVSVMNetConfig.ovs_vlan_config,
    'ovs_ip_config': OVSVMNetConfig.ovs_ip_config,
    'ovs_if_meta': OVSVMNetConfig.ovs_if_meta,
    'ovs_change_mtu': OVSVMNetConfig.ovs_change_mtu,
    'discover_storage_plugins': StoragePluginDiscovery.discover_storage_plugins,
    'storage_plugin_validate': StoragePluginManager.storage_plugin_validate,
    'storage_plugin_getCapabilities': StoragePluginManager.storage_plugin_getCapabilities,
    'storage_plugin_getStorageNames': StoragePluginManager.storage_plugin_getStorageNames,
    'storage_plugin_getInfo': StoragePluginManager.storage_plugin_getInfo,
    'storage_plugin_getAccessGroups': StoragePluginManager.storage_plugin_getAccessGroups,
    'storage_plugin_createAccessGroups': StoragePluginManager.storage_plugin_createAccessGroups,
    'storage_plugin_removeAccessGroups': StoragePluginManager.storage_plugin_removeAccessGroups,
    'storage_plugin_renameAccessGroup': StoragePluginManager.storage_plugin_renameAccessGroup,
    'storage_plugin_addToAccessGroup': StoragePluginManager.storage_plugin_addToAccessGroup,
    'storage_plugin_removeFromAccessGroup': StoragePluginManager.storage_plugin_removeFromAccessGroup,
    'storage_plugin_discover': StoragePluginManager.storage_plugin_discover,
    'storage_plugin_start': StoragePluginManager.storage_plugin_start,
    'storage_plugin_stop': StoragePluginManager.storage_plugin_stop,
    'storage_plugin_refresh': StoragePluginManager.storage_plugin_refresh,
    'storage_plugin_list': StoragePluginManager.storage_plugin_list,
    'storage_plugin_updateSERecords': StoragePluginManager.storage_plugin_updateSERecords,
    'storage_plugin_getStatus': StoragePluginManager.storage_plugin_getStatus,
    'storage_plugin_online': StoragePluginManager.storage_plugin_online,
    'storage_plugin_offline': StoragePluginManager.storage_plugin_offline,
    'storage_plugin_create': StoragePluginManager.storage_plugin_create,
    'storage_plugin_startPresent': StoragePluginManager.storage_plugin_startPresent,
    'storage_plugin_stopPresent': StoragePluginManager.storage_plugin_stopPresent,
    'storage_plugin_resize': StoragePluginManager.storage_plugin_resize,
    'storage_plugin_remove': StoragePluginManager.storage_plugin_remove,
    'storage_plugin_getCloneLimits': StoragePluginManager.storage_plugin_getCloneLimits,
    'storage_plugin_isCloneable': StoragePluginManager.storage_plugin_isCloneable,
    'storage_plugin_clone': StoragePluginManager.storage_plugin_clone,
    'storage_plugin_splitClone': StoragePluginManager.storage_plugin_splitClone,
    'storage_plugin_cloneFromSnap': StoragePluginManager.storage_plugin_cloneFromSnap,
    'storage_plugin_getCurrentClones': StoragePluginManager.storage_plugin_getCurrentClones,
    'storage_plugin_getSnapLimits': StoragePluginManager.storage_plugin_getSnapLimits,
    'storage_plugin_isSnapable': StoragePluginManager.storage_plugin_isSnapable,
    'storage_plugin_createSnap': StoragePluginManager.storage_plugin_createSnap,
    'storage_plugin_createMultiSnap': StoragePluginManager.storage_plugin_createMultiSnap,
    'storage_plugin_isRestorable': StoragePluginManager.storage_plugin_isRestorable,
    'storage_plugin_snapRestore': StoragePluginManager.storage_plugin_snapRestore,
    'storage_plugin_snapRemove': StoragePluginManager.storage_plugin_snapRemove,
    'storage_plugin_getCurrentSnaps': StoragePluginManager.storage_plugin_getCurrentSnaps,
    'storage_plugin_getQoSList': StoragePluginManager.storage_plugin_getQoSList,
    'storage_plugin_setQoS': StoragePluginManager.storage_plugin_setQoS,
    'storage_plugin_getStorageServerInfo': StoragePluginManager.storage_plugin_getStorageServerInfo,
    'storage_plugin_getFileSystemInfo': StoragePluginManager.storage_plugin_getFileSystemInfo,
    'storage_plugin_getFileInfo': StoragePluginManager.storage_plugin_getFileInfo,
    'storage_plugin_listFileSystems': StoragePluginManager.storage_plugin_listFileSystems,
    'storage_plugin_listMountPoints': StoragePluginManager.storage_plugin_listMountPoints,
    'storage_plugin_mount': StoragePluginManager.storage_plugin_mount,
    'storage_plugin_unmount': StoragePluginManager.storage_plugin_unmount,
    'storage_plugin_createFileSystem': StoragePluginManager.storage_plugin_createFileSystem,
    'storage_plugin_resizeFileSystem': StoragePluginManager.storage_plugin_resizeFileSystem,
    'storage_plugin_destroyFileSystem': StoragePluginManager.storage_plugin_destroyFileSystem,
    'storage_plugin_destroy': StoragePluginManager.storage_plugin_destroy,
    'storage_plugin_getFileSystemCloneLimits': StoragePluginManager.storage_plugin_getFileSystemCloneLimits,
    'storage_plugin_getFileSystemSnapLimits': StoragePluginManager.storage_plugin_getFileSystemSnapLimits,
    'storage_plugin_deviceTeardown': StoragePluginManager.storage_plugin_deviceTeardown,
    'storage_plugin_deviceResize': StoragePluginManager.storage_plugin_deviceResize,
    'storage_iSCSI_logoutTarget': StoragePluginManager.storage_iSCSI_logoutTarget,
    'storage_iSCSI_deletePortal': StoragePluginManager.storage_iSCSI_deletePortal,
}


api_v1 = api
