# Copyright (C) 2006-2011 Oracle. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, version 2.  This program is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the GNU
# General Public License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 021110-1307, USA.

from OVSCommons import run_cmd


class Cluster:
    def __init__(self, cfg_file):
        self.config_file = cfg_file

    def add_cluster(self, cluster):
        """Adds cluster to the config file."""
        run_cmd(['/sbin/o2cb',
                 '--config-file=%s' % self.config_file,
                 'add-cluster',
                 cluster['name'],])

    def remove_cluster(self, cluster):
        """Removes cluster from the config file."""
        run_cmd(['/sbin/o2cb',
                 '--config-file=%s' % self.config_file,
                 'remove-cluster',
                 cluster['name'],])

    def add_node(self, node):
        """Adds a node to the cluster in the config file."""
        run_cmd(['/sbin/o2cb',
                 '--config-file=%s' % self.config_file,
                 'add-node',
                 '--ip=%s' % node['ip_address'],
                 '--port=%s' % node['ip_port'],
                 '--number=%s' % node['number'],
                 node['cluster'],
                 node['name'],])

    def remove_node(self, node):
        """Removes a node from the cluster in the config file."""
        run_cmd(['/sbin/o2cb',
                 '--config-file=%s' % self.config_file,
                 'remove-node',
                 node['cluster'],
                 node['name'],])

    def add_heartbeat(self, heartbeat):
        """Adds a heartbeat region to the cluster in the config file."""
        run_cmd(['/sbin/o2cb',
                 '--config-file=%s' % self.config_file,
                 'add-heartbeat',
                 heartbeat['cluster'],
                 heartbeat['region'],])

    def remove_heartbeat(self, heartbeat):
        """Removes a heartbeat region from the cluster in the config file."""
        run_cmd(['/sbin/o2cb',
                 '--config-file=%s' % self.config_file,
                 'remove-heartbeat',
                 heartbeat['cluster'],
                 heartbeat['region'],])

    def heartbeat_mode(self, cluster):
        """Toggles the heartbeat mode between global and local."""
        run_cmd(['/sbin/o2cb',
                 '--config-file=%s' % self.config_file,
                 'heartbeat-mode',
                 cluster['name'],
                 cluster['heartbeat_mode'],])

    def list_cluster(self):
        """Lists all the cluster names in the config file."""
        pass

    def list_cluster(self):
        """Lists all the nodes and heartbeat regions associated with the
        cluster in the config file."""
        pass

    def list_nodes(self):
        """Lists all the nodes associated with the cluster in the config
        file."""
        pass

    def list_heartbeats(self):
        """Lists all the heartbeat regions associated with the cluster in the
        config file."""
        pass

    def register_cluster(self):
        """Registers the cluster with configfs."""
        pass

    def unregister_cluster(self):
        """Unregisters the cluster from configfs."""
        pass

    def start_heartbeat(self):
        """Starts global heartbeat."""
        pass

    def stop_heartbeat(self):
        """Stops global heartbeat."""
        pass

    def cluster_status(self):
        """Returns 0 if cluster online, 1 otherwise."""
        pass
