# Copyright (C) 2006-2011 Oracle. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, version 2.  This program is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the GNU
# General Public License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 021110-1307, USA.

import os
import sha
import string
import random
from OVSConstants import CONFIG_DIR


def encrypt_password(password):
    return sha.new(password).digest().encode("base64").replace("\n", "")


def random_password():
    return ''.join(random.sample(string.letters + string.digits, 8))


def read_password():
    password_dict = {}
    filename = os.path.join(CONFIG_DIR, 'passwdfile')
    passwdfile = open(filename)
    for line in passwdfile:
        username, encrypted_password = line.strip().split(':{SHA}')
        password_dict[username] = encrypted_password
    return password_dict


def write_password(password_dict):
    filename = os.path.join(CONFIG_DIR, 'passwdfile')
    passwdfile = open(filename, 'w')
    try:
        for (username, encrypted_password) in password_dict.items():
            passwdfile.write("%s:{SHA}%s\n" % (username, encrypted_password))
    finally:
        passwdfile.close()

def update_password(username, password):
    password_dict = read_password()
    if username not in password_dict:
        raise Exception('User %s does not exists.' % username)
    password_dict[username] = encrypt_password(password)
    write_password(password_dict)


def add_user(username, encrypted_password=None):
    if not encrypted_password:
        encrypted_password = encrypt_password(random_password())
    password_dict = read_password()
    if username in password_dict:
        raise Exception('User %s already exists.' % username)
    password_dict[username] = encrypted_password
    write_password(password_dict)


def delete_user(username):
    password_dict = read_password()
    try:
        del password_dict[username]
    except KeyError:
        raise Exception("User %s doesn't exist." % username)
    write_password(password_dict)
