#!/usr/bin/env python

# Copyright (C) 2006-2011 Oracle. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, version 2.  This program is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the GNU
# General Public License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 021110-1307, USA.

from os.path import dirname, join
import sys
sys.path.append(join(dirname(__file__), ".."))

import xmlrpclib
import sys
import socket
import re
from ConfigParser import ConfigParser
from OVSConstants import CONFIG_DIR
from utils.xmlrpclib import ServerProxy

def get_local_agt_url():
    config = ConfigParser()
    config.read(join(CONFIG_DIR, "agent.ini"))
    if config.get("security", "ssl").lower().startswith("enable"):
        protocol = "https"
    else:
        protocol = "http"
    agt_url="%s://127.0.0.1:%s" % (protocol, config.get("agent", "port"))
    return agt_url

def main():
    if not re.match(r"https?://.+:[0-9]+", sys.argv[1]):
        agt_url = get_local_agt_url()
        func = sys.argv[1]
        args = sys.argv[2:]
    else:
        agt_url = sys.argv[1]
        func = sys.argv[2]
        args = sys.argv[3:]

    for idx, arg in enumerate(args):
        if arg == "True":
            arg = True
            args[idx] = arg
        elif arg == "False":
            arg = False
            args[idx] = arg

    server = ServerProxy(agt_url)

    server_func_args = [
        ##(server, function_name, parameter_list ),
        (server, func, args),
        ]

    for server, func, args in server_func_args:
        try:
            print "%s %s =>" % (func, args)
            rs = getattr(server, func)(*args)
            if rs != '':
                print "%s" % rs
            sys.exit(0)
        except xmlrpclib.Fault, e:
            print e.faultCode, e.faultString
        except xmlrpclib.ProtocolError, e:
            print e #(url, errcode, errmsg, headers)
        except socket.error, (code, msg):
            print type(code)
            print "socket error! code=%s, msg=%s" % (code, msg)
        except SystemExit:
            raise
        except Exception, e:
            print e.__class__, e
        sys.exit(1)

if __name__ == "__main__":
    main()
